@@ -31,6 +31,8 @@ using Entagged;
 
 namespace Beagle.Filters {
 
+	[PropertyKeywordMapping (Keyword="album", PropertyName="fixme:album", IsKeyword=false)]
+	[PropertyKeywordMapping (Keyword="artist", PropertyName="fixme:artist", IsKeyword=false)]
 	public class FilterMusic : Beagle.Daemon.Filter {
 	
 		public FilterMusic ()
@@ -53,8 +55,9 @@ namespace Beagle.Filters {
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("audio/x-m4a"));
 			AddSupportedFlavor (FilterFlavor.NewFromExtension (".m4p"));
 
-			// OGG
+			// OGG Vorbis
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/ogg"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("audio/x-vorbis+ogg"));
 
 			// Tracker / Amiga Audio
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("audio/x-s3m"));
@@ -63,7 +66,7 @@ namespace Beagle.Filters {
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("audio/x-xm"));
 
 			// ASF / WMA
-			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("video/x-ms-asf"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("audio/x-ms-wma"));
 		}
 
 		private string GetEntaggedMimeType ()
@@ -99,13 +102,13 @@ namespace Beagle.Filters {
 				AddProperty (Beagle.Property.New ("fixme:comment", comment));
 
 			foreach (int track in tag.TrackNumbers)
-				AddProperty (Beagle.Property.NewKeyword ("fixme:tracknumber", track));
+				AddProperty (Beagle.Property.NewUnsearched ("fixme:tracknumber", track));
 
 			foreach (int track in tag.TrackCounts)
-				AddProperty (Beagle.Property.NewKeyword ("fixme:trackcount", track));
+				AddProperty (Beagle.Property.NewUnsearched ("fixme:trackcount", track));
 
 			foreach (int year in tag.Years)
-				AddProperty (Beagle.Property.NewKeyword ("fixme:year", year));
+				AddProperty (Beagle.Property.NewUnsearched ("fixme:year", year));
 
 			foreach (string genre in tag.Genres)
 				AddProperty (Beagle.Property.NewKeyword ("fixme:genre", genre));
