@@ -41,7 +41,8 @@ namespace Beagle.Filters {
 			Fortran_Style,
 			Pascal_Style,
 			Lisp_Style,
-			Matlab_Style
+			Matlab_Style,
+			Shell_Style
 		};
 		
 		protected LangType SrcLangType;
@@ -57,6 +58,7 @@ namespace Beagle.Filters {
 		LineType SrcLineType;
 		string StrConstIdentifier;
 
+		StringBuilder token;
 		public FilterSource ()
 		{
 			// Initialize the linetype member.
@@ -68,6 +70,27 @@ namespace Beagle.Filters {
 
 			SnippetMode = true;
 			OriginalIsText = true;
+			token = new StringBuilder ();
+		}
+
+		// Validate the character and append it to the token,
+		// that will be added to the text-pool of the filter.
+		// Returns: False, if token is not complete enough to be added to the pool.
+		//          True, if it is a valid word that can be added to the pool.
+		private bool AppendToToken (char ch, int index, int length)
+		{
+			if (ch == ' ')
+				return true;
+
+			if (Char.IsLetter (ch) || Char.IsDigit (ch) || ch == '_') {
+				token.Append (ch);
+				if ((index + 1) < length) 
+					return false;
+				else
+					return true;
+			}
+
+			return true;
 		}
 
 		// Tokenize the passed string and add the relevant 
@@ -78,7 +101,7 @@ namespace Beagle.Filters {
  		protected void ExtractTokens (string str)
 		{
 			int index;
-			StringBuilder token = new StringBuilder();
+			token.Length = 0;
 			string splCharSeq = "";
 
 			for (index = 0; index < str.Length; index++) {
@@ -101,7 +124,7 @@ namespace Beagle.Filters {
                                         case "(*":
                                                 if (SrcLineType == LineType.None) {
 							SrcLineType = LineType.BlockComment;
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						    } else 
 							token.Append (splCharSeq);
 						splCharSeq = ""; 
@@ -112,7 +135,7 @@ namespace Beagle.Filters {
 							SrcLineType = LineType.None;
 							token.Append (" ");
 							AppendText (token.ToString());
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						} else if (SrcLineType != LineType.None)
 							token.Append (splCharSeq);
 						splCharSeq = "";
@@ -121,7 +144,7 @@ namespace Beagle.Filters {
                                         case "{":
                                                 if (SrcLineType == LineType.None) {
 							SrcLineType = LineType.BlockComment;
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						    } else 
 							token.Append (splCharSeq);
 						splCharSeq = ""; 
@@ -132,7 +155,7 @@ namespace Beagle.Filters {
 							SrcLineType = LineType.None;
 							token.Append (" ");
 							AppendText (token.ToString());
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						} else if (SrcLineType != LineType.None)
 							token.Append (splCharSeq);
 						splCharSeq = "";
@@ -140,7 +163,7 @@ namespace Beagle.Filters {
 					case "//":
 						if (SrcLineType == LineType.None) {
 							SrcLineType = LineType.SingleLineComment;
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						} else
 							token.Append (splCharSeq);
 						splCharSeq = "";
@@ -149,7 +172,7 @@ namespace Beagle.Filters {
 					case "/*":
 						if (SrcLineType == LineType.None) {
 							SrcLineType = LineType.BlockComment;
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						} else 
 							token.Append (splCharSeq);
 						splCharSeq = "";
@@ -160,19 +183,20 @@ namespace Beagle.Filters {
 							SrcLineType = LineType.None;
 							token.Append (" ");
 							AppendText (token.ToString());
-							token.Remove (0, token.Length);
+							token.Length = 0;
 						} else if (SrcLineType != LineType.None)
 							token.Append (splCharSeq);
 						splCharSeq = "";
 						break;
 					}
-				} else if ((str[index] == '#' && SrcLangType == LangType.Python_Style) ||
+				} else if ((str[index] == '#' && (SrcLangType == LangType.Python_Style ||
+								  SrcLangType == LangType.Shell_Style)) ||
 					   (str[index] == '!' && SrcLangType == LangType.Fortran_Style) ||
 					   (str[index] == ';' && SrcLangType == LangType.Lisp_Style) ||
 					   (str[index] == '%' && SrcLangType == LangType.Matlab_Style)) {
 					if (SrcLineType == LineType.None) {
 						SrcLineType = LineType.SingleLineComment;
-						token.Remove (0, token.Length);
+						token.Length = 0;
 					} else
 						token.Append (str[index]);
 				}
@@ -196,11 +220,11 @@ namespace Beagle.Filters {
 						SrcLineType = LineType.None;
 						token.Append (" ");
 						AppendText (token.ToString());
-						token.Remove (0, token.Length);
+						token.Length = 0;
 					} else {
 						StrConstIdentifier = str.Substring (index, 3);
 						SrcLineType = LineType.StringConstant;
-						token.Remove (0, token.Length);
+						token.Length = 0;
 						index += 2;
 					}
 					       
@@ -210,7 +234,8 @@ namespace Beagle.Filters {
 				else if (SrcLangType == LangType.Lisp_Style && str[index] == '\'') {
 					continue;
 				}
-				else if (str[index] == '\"' || str[index] == '\'') {
+				else if (str[index] == '\"' || str[index] == '\'' ||
+					 (str[index] == '`' && SrcLangType == LangType.Shell_Style)) {
 
 					if (SrcLineType == LineType.StringConstant &&
 					    StrConstIdentifier.Length == 1 &&
@@ -218,12 +243,12 @@ namespace Beagle.Filters {
 						SrcLineType = LineType.None;
 						token.Append (" ");
 						AppendText (token.ToString());
-						token.Remove (0, token.Length);
+						token.Length = 0;
 
 					} else if (SrcLineType == LineType.None) {
 						StrConstIdentifier = str.Substring (index, 1);
 						SrcLineType = LineType.StringConstant;
-						token.Remove (0, token.Length);
+						token.Length = 0;
 					} else
 						token.Append (str[index]);
 					splCharSeq = "";
@@ -233,12 +258,8 @@ namespace Beagle.Filters {
 					token.Append (str[index]);
 					splCharSeq = "";
 
-				} else if (SrcLineType == LineType.None) {
-					if (Char.IsLetter (str[index]) ||
-					    Char.IsDigit (str[index]) ||
-					    str[index] == '_') {
-						token.Append (str[index]);
-					} else {
+				} else if (SrcLineType == LineType.None) { 
+					if (AppendToToken (str[index], index, str.Length)) {			       
 						if (SrcLangType == LangType.Lisp_Style) {
 
 							// Lisp identifiers: letters, digits, and:
@@ -253,20 +274,22 @@ namespace Beagle.Filters {
 								continue;
 							}
 						}
-						token = token.Replace(" ", "");
+						//token = token.Replace(" ", "");
 						if (token.Length > 0) {
-							string tok = token.ToString(); 
+							string tok;
 							if (SrcLangType == LangType.Fortran_Style)
 								tok = token.ToString().ToLower();
-							
+							else
+								tok = token.ToString ();
 							if (!KeyWordsHash.Contains (tok)) {
-								token.Append (" ");
-								if (!Char.IsDigit (token[0]))
-									AppendText (token.ToString());
+								if (!Char.IsDigit (token[0])) {
+									AppendText (tok);
+									AppendWhiteSpace ();
+								}
 							}
 						}
 						// reset the token
-						token.Remove (0, token.Length);
+						token.Length = 0;
 					}
 					splCharSeq = "";
 				}
