@@ -168,9 +168,17 @@ namespace HtmlAgilityPack
 		}
 	}
 	
+
+	abstract class StreamAsArray {
+		public abstract bool Eof (int index);
+		public abstract char this [int index] { get;}
+		public abstract string Substring (int startindex, int length);
+		public abstract int FullLength { get;}
+	}
+		
 	// SLIM: creating this class to wrap around a textreader
 	//	 to emulate ReadToEnd () behaviour
-	class StreamAsArray {
+	class ImplStreamAsArray : StreamAsArray {
 		private StreamReader _reader;
 		private int _length;
 		private int _position;
@@ -179,7 +187,7 @@ namespace HtmlAgilityPack
 		private char[] _buf_current; // but, this is cleaner
 		private int _block_size;
 		
-		public StreamAsArray (StreamReader r)
+		public ImplStreamAsArray (StreamReader r)
 		{
 			_reader = r;
 			_length = 0;
@@ -209,7 +217,7 @@ namespace HtmlAgilityPack
 			HtmlDocument.Debug ("[" + new string (_buf_current, 0, num_read) + "]");
 		}
 		
-		public bool Eof (int index) {
+		public override bool Eof (int index) {
 			if (_eof)
 				return (index == _length);
 			else {
@@ -223,7 +231,7 @@ namespace HtmlAgilityPack
 			}
 		}
 		
-		public new char this[int index] {
+		public override char this[int index] {
 			get {
 				if (index >= _position && 
 				    index < _position + _block_size)
@@ -288,7 +296,7 @@ namespace HtmlAgilityPack
 			tmp = null;
 		}
 
-		public string Substring (int startindex, int length)
+		public override string Substring (int startindex, int length)
 		{
 			if (length == 0) {
 				HtmlDocument.Debug ("substring:" + startindex + " " + length + " " + _position + ":");
@@ -316,24 +324,56 @@ namespace HtmlAgilityPack
 		}
 
 		// FIXME: Is this costly ?
-		public int FullLength {
+		public override int FullLength {
 			get {
 				return (int)_reader.BaseStream.Length;
 			}
 		}
 	}
 
+	// A dummy StreamAsArray wrapper around a string
+	class DummyStreamAsArray : StreamAsArray {
+		private string _base_string;
+		private int _length;
+
+		public DummyStreamAsArray(string str)
+		{
+			_base_string = str;
+			_length = str.Length;
+		}
+
+		public override bool Eof(int index)
+		{
+			return (index >= _length);
+		}
+
+		public new char this[int index] {
+			get { return _base_string [index]; }
+		}
+
+		public override string Substring (int startindex, int length)
+		{
+			return _base_string.Substring (startindex, length);
+		}
+
+		public override int FullLength {
+			get { return _length; }
+		}
+	}
+
 	/// <summary>
 	/// Represents a complete HTML document.
 	/// </summary>
 	public class HtmlDocument: IXPathNavigable
 	{
 		// SLIM: Make the parser event driven
-		// callbacks for FilterHtml
-		public delegate void NodeHandler (HtmlNode node);
+		// callback for FilterHtml
+		// return value is a way for the callback to signal to continue or stop parsing
+		public delegate bool NodeHandler (HtmlNode node);
 		public NodeHandler ReportNode;
 		// misnomer ... should be called event_driven_mode
 		private bool _streammode = false;
+		private bool _stop_parsing = false;
 
 		internal static readonly string HtmlExceptionRefNotChild = "Reference node must be a child of this node";
 		internal static readonly string HtmlExceptionUseIdAttributeFalse = "You need to set UseIdAttribute property to true to enable this feature";
@@ -606,15 +646,17 @@ namespace HtmlAgilityPack
 			if (sr != null)
 			{
 				_streamencoding = sr.CurrentEncoding;
+				_text = new ImplStreamAsArray (sr);
 			}
 			else
 			{
 				_streamencoding = null;
+				// Expensive, but cannot avoid since TextReader doesnt have any length of the underlying data
+				_text = new DummyStreamAsArray (reader.ReadToEnd());
 			}
 			_declaredencoding = null;
 
 			// SLIM: _text = reader.ReadToEnd();
-			_text = new StreamAsArray (sr);
 			_documentnode = CreateNode(HtmlNodeType.Document, 0);
 
 			// this is a hack, but it allows us not to muck with the original parsing code
@@ -876,15 +918,17 @@ namespace HtmlAgilityPack
 				    // void on purpose
 				}
 				_streamencoding = sr.CurrentEncoding;
+				_text = new ImplStreamAsArray (sr);
 			}
 			else
 			{
 				_streamencoding = null;
+				// Expensive, but cannot avoid since TextReader doesnt have any length of the underlying data
+				_text = new DummyStreamAsArray (reader.ReadToEnd());
 			}
 			_declaredencoding = null;
 
 			// SLIM: _text = reader.ReadToEnd();
-			_text = new StreamAsArray (sr);
 			_documentnode = CreateNode(HtmlNodeType.Document, 0);
 			Parse();
 
@@ -1352,7 +1396,7 @@ namespace HtmlAgilityPack
 			_index = 0;
 			PushNodeStart(HtmlNodeType.Text, 0);
 			// SLIM: while (_index<_text.Length)
-			while (! _text.Eof (_index))
+			while (! _stop_parsing && ! _text.Eof (_index))
 			{
 				_c = _text[_index];
 				IncrementPosition();
@@ -1658,7 +1702,7 @@ namespace HtmlAgilityPack
 										_currentnode._outerstartindex + _currentnode._outerlength);
 									script._outerlength = _index-1 - script._outerstartindex;
 									if (_streammode && ReportNode != null)
-										ReportNode (script);
+										_stop_parsing = ! ReportNode (script);
 									else
 										_currentnode.AppendChild(script);
 									Debug ("Found script: [" + script.InnerText + "]");
@@ -1848,7 +1892,7 @@ namespace HtmlAgilityPack
 
 			//SLIM: inform caller
 			if (_streammode && ReportNode != null)
-				ReportNode (_currentnode);
+				_stop_parsing = ! ReportNode (_currentnode);
 
 			if (_debug) {
 				if (_currentnode._nodetype == HtmlNodeType.Text)
