@@ -87,9 +87,7 @@ namespace Beagle.Util {
 			get { return utterances; }
 		}
 
-		protected void AddUtterance (DateTime timestamp,
-					     String   who,
-					     String   text)
+		protected void AddUtterance (DateTime timestamp, string who, string text)
 		{
 			Utterance utt = new Utterance ();
 			utt.Timestamp = timestamp;
@@ -133,48 +131,6 @@ namespace Beagle.Util {
 
 		public const string MimeType = "beagle/x-gaim-log";
 
-		private static string StripTags (string line, StringBuilder builder)
-		{
-			int first = line.IndexOf ('<');
-			if (first == -1)
-				return line;
-			
-			builder.Length = 0;
-
-			int i = 0;
-			while (i < line.Length) {
-				
-				int j;
-				if (first == -1) {
-					j = line.IndexOf ('<', i);
-				} else {
-					j = first;
-					first = -1;
-				}
-				
-				int k = -1;
-				if (j != -1) {
-					k = line.IndexOf ('>', j);
-					
-					// If a "<" is unmatched, preserve it, and the
-					// rest of the line
-					if (k == -1)
-						j = -1;
-				}
-				
-				if (j == -1) {
-					builder.Append (line, i, line.Length - i);
-					break;
-				}
-				
-				builder.Append (line, i, j-i);
-				
-				i = k+1;
-			}
-			
-			return builder.ToString ();
-		}
-
 		///////////////////////////////////////
 
 		public GaimLog (FileInfo file, TextReader reader) : base ("gaim", file, reader)
@@ -244,13 +200,24 @@ namespace Beagle.Util {
 				AppendToPreviousUtterance (line);
 				return;
 			}
-			string whenStr = line.Substring (1, j-1);
-			string[] whenSplit = whenStr.Split (':');
-			int hour, minute, second;
+
+			// Gaim 2.0
+			// The new version of Gaim adds AM or PM right after the time
+			// 1.x: (19:07:07)
+			// 2.0: (19:07:07 AM)
+
+			string when = line.Substring (1, j-1);
+			DateTime timestamp;
+			
 			try {
-				hour   = int.Parse (whenSplit [0]);
-				minute = int.Parse (whenSplit [1]);
-				second = int.Parse (whenSplit [2]);
+				DateTime time = DateTime.Parse (when);
+				
+				timestamp = new DateTime (StartTime.Year, StartTime.Month, StartTime.Day,
+						time.Hour, time.Minute, time.Second);
+
+				// Try to deal with time wrapping around.
+				if (timestamp < EndTime)
+					timestamp.AddDays (1);
 			} catch {
 				// If something goes wrong, this line probably
 				// spills over from the previous one.
@@ -259,25 +226,16 @@ namespace Beagle.Util {
 			}
 
 			line = line.Substring (j+2);
-
-			// FIXME: this is wrong --- since we just get a time,
-			// the date gets set to 'now'
-			DateTime when = new DateTime (StartTime.Year,
-						      StartTime.Month,
-						      StartTime.Day,
-						      hour, minute, second);
-
-			// Try to deal with time wrapping around.
-			while (when < EndTime)
-				when = when.AddDays (1);
-
+						
+			// Extract the alias
+			// FIXME: This will break if there is a ':' in the nickname
 			int i = line.IndexOf (':');
 			if (i == -1)
 				return;
 			string alias = line.Substring (0, i);
 			string text = line.Substring (i+2);
 
-			AddUtterance (when, alias, text);
+			AddUtterance (timestamp, alias, text);
 
 			return;
 		}
@@ -300,8 +258,8 @@ namespace Beagle.Util {
 				
 			while ((line = TextReader.ReadLine ()) != null) {
 				if (isHtml)
-					line = StripTags (line, builder);
-				
+					line = StringFu.StripTags (line, builder);
+			
 				ProcessLine (line);
 			}
 		}
