@@ -34,7 +34,7 @@ namespace Beagle.Util {
 
 		private PathFinder () { }
 
-		static private string PkgLibDir {
+		static public string PkgLibDir {
 			get { return ExternalStringsHack.PkgLibDir; }
 		}
 
@@ -74,6 +74,7 @@ namespace Beagle.Util {
 							throw new Exception ("Couldn't get HOME or BEAGLE_HOME");
 						if (home_dir.EndsWith ("/"))
 							home_dir = home_dir.Remove (home_dir.Length - 1, 1);
+						home_dir = Path.GetFullPath (home_dir);
 						if (! Directory.Exists (home_dir))
 							throw new Exception ("Home directory '"+home_dir+"' doesn't exist");
 					}
@@ -81,6 +82,9 @@ namespace Beagle.Util {
 				
 				return home_dir;
 			}
+
+			// You probably don't want to do this.
+			set { lock (home_dir_lock) { home_dir = value; } }
 		}
 
 		// The storage directory is the place where beagle stores its private data.
@@ -96,14 +100,17 @@ namespace Beagle.Util {
 
 						if (storage_dir == null)
 							storage_dir = Path.Combine (HomeDir, ".beagle");
-						else if (storage_dir.EndsWith ("/"))
-							storage_dir = storage_dir.Remove (storage_dir.Length - 1, 1);
+						else {
+							if (storage_dir.EndsWith ("/"))
+								storage_dir = storage_dir.Remove (storage_dir.Length - 1, 1);
+							storage_dir = Path.GetFullPath (storage_dir);
+						}
 
 						if (! Directory.Exists (storage_dir)) {
 							Directory.CreateDirectory (storage_dir);
 							// Make sure that the directory is only
 							// readable by the owner.
-							Mono.Posix.Syscall.chmod (storage_dir, (Mono.Posix.FileMode) 448); // 448 == 0700
+							Mono.Unix.Native.Syscall.chmod (storage_dir, (Mono.Unix.Native.FilePermissions) 448); // 448 == 0700
 						}
 					}
 				}
@@ -112,6 +119,75 @@ namespace Beagle.Util {
 			}
 		}
 
+		static string remote_storage_dir;
+		static public string GetRemoteStorageDir (bool create)
+		{
+			if (remote_storage_dir == null) {
+				if ((! SystemInformation.IsPathOnBlockDevice (PathFinder.StorageDir) && Conf.Daemon.IndexSynchronization) ||
+				    Environment.GetEnvironmentVariable ("BEAGLE_SYNCHRONIZE_LOCALLY") != null) {
+					string index_pointer = Path.Combine (StorageDir, "remote_storage_dir");
+
+					if (File.Exists (index_pointer)) {
+						StreamReader r = new StreamReader (index_pointer);
+						remote_storage_dir = r.ReadLine ();
+						r.Close ();
+
+						if (!Directory.Exists (remote_storage_dir))
+							remote_storage_dir = null;
+					}
+
+					if (create) {
+						if (remote_storage_dir == null) {
+							do {
+								string p = String.Format ("beagle-{0}-{1}", Environment.GetEnvironmentVariable ("USER"),
+											  Guid.NewGuid ().ToString ());
+
+								remote_storage_dir = Path.Combine (Path.GetTempPath (), p);
+							} while (Directory.Exists (remote_storage_dir) || File.Exists (remote_storage_dir));
+
+							StreamWriter w = new StreamWriter (index_pointer);
+							w.WriteLine (remote_storage_dir);
+							w.Close ();
+
+
+						}
+
+						if (! Directory.Exists (remote_storage_dir)) {
+							Directory.CreateDirectory (remote_storage_dir);
+							// Make sure that the directory is only
+							// readable by the owner.
+							Mono.Unix.Native.Syscall.chmod (remote_storage_dir, (Mono.Unix.Native.FilePermissions) 448); // 448 == 0700
+						}
+					}
+				} else
+					remote_storage_dir = StorageDir;
+			}
+
+			return remote_storage_dir;
+		}
+
+		// The directory where beagle stores its indexes
+		// Fun fact #1: It will be synchronized locally if PathFinder.HomeDir
+		// is on a non-block device, or if BEAGLE_SYNCHRONIZE_LOCALLY is set.
+		static string index_dir;
+		static public string IndexDir {
+			get { 
+				if (index_dir == null) {
+					index_dir = Path.Combine (GetRemoteStorageDir (true), "Indexes");
+					
+					if (! Directory.Exists (index_dir)) {
+						Directory.CreateDirectory (index_dir);
+
+						// Make sure that the directory is only readable by the owner. 
+						// Required when using index synchronization as then it resides in /tmp
+						Mono.Unix.Native.Syscall.chmod (index_dir, (Mono.Unix.Native.FilePermissions) 448); // 448 == 0700
+					}
+				}
+				
+				return index_dir;
+			}
+		}
+
 		static public string LogDir {
 			get {
 				string dir = Path.Combine (StorageDir, "Log");
@@ -120,72 +196,5 @@ namespace Beagle.Util {
 				return dir;
 			}
 		}
-
-		static private string AppDir {
-			get {
-				string dir = Path.Combine (StorageDir, "App");
-				if (! Directory.Exists (dir))
-					Directory.CreateDirectory (dir);
-				return dir;
-			}
-		}
-
-		// We probably shouldn't expose this.  Use it only for good, not
-		// for evil.
-		static public string AppDataFileName (string appName, string dataName)
-		{
-			// FIXME: should make sure appName & dataName don't
-			// contain evil characters.
-			return Path.Combine (AppDir,
-					     string.Format ("{0}_-_-{1}", appName, dataName));
-		}
-
-		
-		static public bool HaveAppData (string appName, string dataName)
-		{
-			return File.Exists (AppDataFileName (appName, dataName));
-		}
-
-		static public Stream ReadAppData (string appName, string dataName)
-		{
-			return new FileStream (AppDataFileName (appName, dataName),
-					       FileMode.Open,
-					       FileAccess.Read);
-		}
-
-		static public string ReadAppDataLine (string appName, string dataName)
-		{
-			if (! HaveAppData (appName, dataName))
-				return null;
-
-			StreamReader sr = new StreamReader (ReadAppData (appName, dataName));
-			string line = sr.ReadLine ();
-			sr.Close ();
-
-			return line;
-		}
-
-		static public Stream WriteAppData (string appName, string dataName)
-		{
-			return new FileStream (AppDataFileName (appName, dataName),
-					       FileMode.Create,
-					       FileAccess.Write);
-		}
-
-		static public void WriteAppDataLine (string appName, string dataName, string line)
-		{
-			if (line == null) {
-				string fileName = AppDataFileName (appName, dataName); 
-				if (File.Exists (fileName))
-					File.Delete (fileName);
-				return;
-			}
-
-			StreamWriter sw = new StreamWriter (WriteAppData (appName, dataName));
-			sw.WriteLine (line);
-			sw.Close ();
-		}
 	}
-	
-
 }
