@@ -31,7 +31,7 @@ using System.IO;
 using System.Text;
 using System.Xml;
 
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Beagle.Util {
 
@@ -39,6 +39,8 @@ namespace Beagle.Util {
 
 		private StringFu () { } // class is static
 
+		public const string UnindexedNamespace = "_unindexed:";
+
 		private const String timeFormat = "yyyyMMddHHmmss";
 
 		static public string DateTimeToString (DateTime dt)
@@ -46,6 +48,16 @@ namespace Beagle.Util {
 			return dt.ToString (timeFormat);
 		}
 
+		static public string DateTimeToYearMonthString (DateTime dt)
+		{
+			return dt.ToString ("yyyyMM");
+		}
+
+		static public string DateTimeToDayString (DateTime dt)
+		{
+			return dt.ToString ("dd");
+		}
+
                 static public DateTime StringToDateTime (string str)
                 {
 			if (str == null || str == "")
@@ -203,6 +215,12 @@ namespace Beagle.Util {
 
 			return builder.ToString ();
 		}
+
+		public static string UrlFuzzyDivide (string url)
+		{
+			int protocol_index = url.IndexOf ("://");
+			return FuzzyDivide (url.Substring (protocol_index + 3));
+		}
 		
 		// Match strings against patterns that are allowed to contain
 		// glob-style * wildcards.
@@ -210,6 +228,9 @@ namespace Beagle.Util {
 		// and probably will fail for weird corner cases.
 		static public bool GlobMatch (string pattern, string str)
 		{
+			if (pattern == null || str == null)
+				return false;
+
 			if (pattern == "*")
 				return true;
 			else if (pattern.StartsWith ("**"))
@@ -300,7 +321,7 @@ namespace Beagle.Util {
 			int i = 0, pos = 0;
 			while ((i = str.IndexOf ('%', pos)) != -1) {
 				pos = i;
-				char unescaped = UriFu.HexUnescape (str, ref pos);
+				char unescaped = Uri.HexUnescape (str, ref pos);
 				str = str.Remove (i, 3);
 				str = str.Insert (i, new String(unescaped, 1));
 				pos -= 2;
@@ -461,19 +482,26 @@ namespace Beagle.Util {
 			return new string (char_array);
 		}
 		
-		static public int CountWords (string str, int max_words)
+		// Words of less than min_word_length characters are not counted
+		static public int CountWords (string str, int max_words, int min_word_length)
 		{
 			if (str == null)
 				return 0;
 
 			bool last_was_white = true;
 			int words = 0;
+			int word_start_pos = -1;
+
 			for (int i = 0; i < str.Length; ++i) {
 				if (Char.IsWhiteSpace (str [i])) {
+					// if just seen word is too short, ignore it
+					if (! last_was_white && (i - word_start_pos < min_word_length))
+						--words;
 					last_was_white = true;
 				} else {
 					if (last_was_white) {
 						++words;
+						word_start_pos = i;
 						if (max_words > 0 && words >= max_words)
 							break;
 					}
@@ -484,6 +512,11 @@ namespace Beagle.Util {
 			return words;
 		}
 
+		static public int CountWords (string str, int max_words)
+		{
+			return CountWords (str, max_words, -1);
+		}
+
 		static public int CountWords (string str)
 		{
 			return CountWords (str, -1);
@@ -505,5 +538,165 @@ namespace Beagle.Util {
 
 			return path;
 		}
+
+		// This method will translate an email address like
+		// "john.doe+spamtrap@foo.com" to "john doe spamtrap foo"
+		//
+		// FIXME: Maybe we should only do the username part?  Ie,
+		// "john doe spamtrap"?  That way searching for "foo" won't
+		// turn up *everything*
+		static public string SanitizeEmail (string email)
+		{
+			char[] replace_array = { '@', '.', '-', '_', '+' };
+			string[] tlds = { "com", "net", "org", "edu", "gov", "mil" }; // Just the Big Six
+
+			if (email == null)
+				return null;
+
+			email = email.ToLower ();
+
+			string[] tmp = email.Split (replace_array);
+			email = String.Join (" ", tmp);
+
+			foreach (string tld in tlds) {
+				if (email.EndsWith (" " + tld)) {
+					email = email.Substring (0, email.Length - 4);
+					break;
+				}
+			}
+
+			return email;
+		}
+
+		/**
+		 * expands environment variables in a string e.g.
+		 * folders=$HOME/.kde/share/...
+		 */
+		public static string ExpandEnvVariables (string path)
+		{
+			int dollar_pos = path.IndexOf ('$');
+			if (dollar_pos == -1)
+				return path;
+			
+			System.Text.StringBuilder sb = 
+				new System.Text.StringBuilder ( (dollar_pos == 0 ? "" : path.Substring (0, dollar_pos)));
+			
+			while (dollar_pos != -1 && dollar_pos + 1 < path.Length) {
+				// FIXME: kconfigbase.cpp contains an additional case, $(expression)/.kde/...
+				// Ignoring such complicated expressions for now. Volunteers ;) ?
+				int end_pos = dollar_pos;
+				if (path [dollar_pos + 1] != '$') {
+					string var_name;
+					end_pos ++;
+					if (path [end_pos] == '{') {
+						while ((end_pos < path.Length) && 
+						       (path [end_pos] != '}'))
+							end_pos ++;
+						end_pos ++;
+						var_name = path.Substring (dollar_pos + 2, end_pos - dollar_pos - 3);
+					} else {
+						while ((end_pos < path.Length) &&
+						       (Char.IsNumber (path [end_pos]) ||
+							Char.IsLetter (path [end_pos]) ||
+							path [end_pos] == '_'))
+							end_pos ++;
+						var_name = path.Substring (dollar_pos + 1, end_pos - dollar_pos - 1);
+					}
+					string value_env = null;
+					if (var_name != String.Empty)
+						value_env = Environment.GetEnvironmentVariable (var_name);
+					if (value_env != null) {
+						sb.Append (value_env);
+					}
+					// else, no environment variable with that name exists. ignore
+				}else // else, ignore the first '$', second one will be expanded
+					end_pos ++;
+				if (end_pos >= path.Length)
+					break;
+				dollar_pos = path.IndexOf ('$', end_pos);
+				if (dollar_pos == -1) {
+					sb.Append (path.Substring (end_pos));
+				} else {
+					sb.Append (path.Substring (end_pos, dollar_pos - end_pos));
+				}
+			}
+
+			return sb.ToString ();
+		}
+
+		public static string StripTags (string line, StringBuilder builder)
+		{
+			int first = line.IndexOf ('<');
+			if (first == -1)
+				return line;
+			
+			builder.Length = 0;
+
+			int i = 0;
+			while (i < line.Length) {
+				
+				int j;
+				if (first == -1) {
+					j = line.IndexOf ('<', i);
+				} else {
+					j = first;
+					first = -1;
+				}
+				
+				int k = -1;
+				if (j != -1) {
+					k = line.IndexOf ('>', j);
+					
+					// If a "<" is unmatched, preserve it, and the
+					// rest of the line
+					if (k == -1)
+						j = -1;
+				}
+				
+				if (j == -1) {
+					builder.Append (line, i, line.Length - i);
+					break;
+				}
+				
+				builder.Append (line, i, j-i);
+				
+				i = k+1;
+			}
+			
+			return builder.ToString ();
+		}
+
+		public static string StripTags (string line)
+		{
+			StringBuilder sb = new StringBuilder ();
+			return StripTags (line, sb);
+		}
+
+	}
+
+	public class HtmlRemovingReader : TextReader {
+		private TextReader reader;	
+		private StringBuilder sb;
+	    
+		public HtmlRemovingReader (TextReader reader)
+		{
+			this.reader = reader;
+			sb = new StringBuilder ();
+		}
+
+		public override string ReadLine ()
+		{
+			string line = reader.ReadLine ();
+			if (line == null)
+				return null;
+
+			sb.Length = 0;
+			return StringFu.StripTags (line, sb);
+		}
+
+		public override void Close ()
+		{
+			reader.Close ();
+		}
 	}
 }
