@@ -42,119 +42,36 @@ namespace Beagle.Util {
 
 		static public Uri UriStringToUri (string path)
 		{
-			// Decode our pre-encoded 'odd' characters into their real values
-			int i = 0, pos = 0;
-			while ((i = path.IndexOf ('%', pos)) != -1) {
-				pos = i;
-				char unescaped = UriFu.HexUnescape (path, ref pos);
-				if (unescaped < '!' || unescaped > '~') {
-					path = path.Remove (i, 3);
-					path = path.Insert (i, new String(unescaped, 1));
-					pos -= 2;
-				}
-			}
-		
-			// Paths from the file:// indexer need (re)quoting. For example,
-			// valid characters such as @ need to be converted to their hex
-			// values.
-			if (path.StartsWith ("file://")) {
-				// Remove the file:// prefix
-				path = path.Substring (7);
-
-				return PathToFileUri (path);
-			}
-			
-			// Currently, no other protocols need extra processing
+			// Our current hackery attempts to serialize Uri strings in
+			// escaped and constructable form, so we don't require any
+			// extra processing on deserialization right now.
 			return new Uri (path, true);
 		}
 
 		static public String UriToSerializableString (Uri uri)
 		{
 			int i;
-			string ret;
+			string path;
 			StringBuilder builder = new StringBuilder ();
-			
-			// The ToString() of a file:// URI is not always representative of
-			// what it was constructed from. For example, it will return a
-			// # (which was inputted as %23) as %23, whereas the more standard
-			// behaviour for other escaped-characters is to return them as
-			// their actual character. (e.g. %40 gets returned as @)
-			// On the other hand, the LocalPath of a file:// URI does seem to
-			// return the literal # so we use that instead.
+
 			if (uri.IsFile)
-				ret = Uri.UriSchemeFile + Uri.SchemeDelimiter + uri.LocalPath;
+				path = Uri.UriSchemeFile + Uri.SchemeDelimiter
+					+ StringFu.HexEscape (uri.LocalPath);
 			else
-				ret = uri.ToString ();
+				path = uri.ToString ();
 
 			// XmlSerializer is happy to serialize 'odd' characters, but doesn't
 			// like to deserialize them. So we encode all 'odd' characters now.
-			for (i = 0; i < ret.Length; i++)
-				if ((ret [i] < '!') || (ret [i] > '~' && ret [i] < 256))
-					builder.Append (Uri.HexEscape (ret [i]));
+			for (i = 0; i < path.Length; i++)
+				if ((path [i] < '!') || (path [i] > '~' && path [i] < 256))
+					builder.Append (Uri.HexEscape (path [i]));
 				else
-					builder.Append (ret [i]);
+					builder.Append (path [i]);
 
-			return builder.ToString ();
-		}
-
-		// Stolen from Mono SVN 20050319
-		// Fixes bug where non-ASCII characters couldn't be decoded
-		// FIXME: Go back to using Uri.HexUnescape when new Mono 1.1.5+ is 
-		// readily available.
-		public static char HexUnescape (string pattern, ref int index) 
-		{
-			if (pattern == null) 
-				throw new ArgumentException ("pattern");
-				
-			if (index < 0 || index >= pattern.Length)
-				throw new ArgumentOutOfRangeException ("index");
-
-			if (!Uri.IsHexEncoding (pattern, index))
-				return pattern [index++];
-
-			int stage = 0;
-			int c = 0;
-			int b = 0;
-			bool looped = false;
-			do {
-				index++;
-				int msb = Uri.FromHex (pattern [index++]);
-				int lsb = Uri.FromHex (pattern [index++]);
-				b = (msb << 4) + lsb;
-				if (!Uri.IsHexEncoding (pattern, index)) {
-					if (looped)
-						c += (b - 0x80) << ((stage - 1) * 6);
-					else
-						c = b;
-					break;
-				} else if (stage == 0) {
-					if (b < 0xc0)
-						return (char) b;
-					else if (b < 0xE0) {
-						c = b - 0xc0;
-						stage = 2;
-					} else if (b < 0xF0) {
-						c = b - 0xe0;
-						stage = 3;
-					} else if (b < 0xF8) {
-						c = b - 0xf0;
-						stage = 4;
-					} else if (b < 0xFB) {
-						c = b - 0xf8;
-						stage = 5;
-					} else if (b < 0xFE) {
-						c = b - 0xfc;
-						stage = 6;
-					}
-					c <<= (stage - 1) * 6;
-				} else {
-					c += (b - 0x80) << ((stage - 1) * 6);
-				}
-				stage--;
-				looped = true;
-			} while (stage > 0);
+			if (uri.IsFile)
+				builder.Append (uri.Fragment);
 			
-			return (char) c;
+			return builder.ToString ();
 		}
 
 		static public String LocalPathFromUri (Uri uri)
@@ -171,6 +88,18 @@ namespace Beagle.Util {
 
 		//////////////////////////////////
 
+		static public bool Equals (Uri uri1, Uri uri2)
+		{
+			return uri1.ToString () == uri2.ToString ();
+		}
+
+		static public int Compare (Uri uri1, Uri uri2)
+		{
+			return String.Compare (uri1.ToString (), uri2.ToString ());
+		}
+
+		//////////////////////////////////
+
 		public class Comparer : IComparer
 		{
 			public int Compare(object uri1, object uri2)
