@@ -260,8 +260,9 @@ namespace Beagle.Daemon
 				}
 			}
 
-			// Set the IO priority to idle so we don't slow down the system
-			IoPriority.SetIdle ();
+			// Set the IO priority so we don't slow down the system
+			if (! IoPriority.SetIdle ())
+				IoPriority.SetIoPriority (7);
 			
 			driver = new LuceneIndexingDriver (arg_output, false);
 			driver.TextCache = (arg_cache_text) ? new TextCache (arg_output) : null;
@@ -327,12 +328,12 @@ namespace Beagle.Daemon
 						if (arg_recursive)
 							foreach (DirectoryInfo subdir in DirectoryWalker.GetDirectoryInfos (dir))
 								if (!Ignore (subdir)
-								    && !FileSystem.IsSymLink (subdir.FullName))
+								    && !FileSystem.IsSpecialFile (subdir.FullName))
 									pending_directories.Enqueue (subdir);
 					
 						foreach (FileInfo file in DirectoryWalker.GetFileInfos (dir))
 							if (!Ignore (file)
-							    && !FileSystem.IsSymLink (file.FullName))
+							    && !FileSystem.IsSpecialFile (file.FullName))
 								pending_files.Enqueue (file);
 					
 					} catch (DirectoryNotFoundException e) {}
@@ -589,7 +590,7 @@ namespace Beagle.Daemon
 			if (file.Name.StartsWith ("."))
 				return true;
 
-			if (FileSystem.IsSymLink (file.FullName))
+			if (FileSystem.IsSpecialFile (file.FullName))
 				return true;
 
 			if (allowed_patterns.Count > 0) {
