@@ -238,7 +238,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				
 				Logger.Log.Debug ("{0}: Finished indexing {1} messages", this.folder_name, this.indexed_count);
 
-				this.MboxLastOffset = offset;
+				if (offset >= 0)
+					this.MboxLastOffset = offset;
 				this.CrawlFinished ();
 
 				return false;
@@ -249,8 +250,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		public override Indexable GetNextIndexable ()
 		{
 			using (GMime.Message message = this.mbox_parser.ConstructMessage ()) {
-				++this.count;
-
 				// Work around what I think is a bug in GMime: If you
 				// have a zero-byte file or seek to the end of a
 				// file, parser.Eos () will return true until it
@@ -261,9 +260,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				//
 				// Check if its empty by seeing if the Headers
 				// property is null or empty.
-				if (message.Headers == null || message.Headers == "")
+				if (message == null || message.Headers == null || message.Headers == "")
 					return null;
 				
+				++this.count;
+
 				string x_evolution = message.GetHeader ("X-Evolution");
 				if (x_evolution == null || x_evolution == "") {
 					Logger.Log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
@@ -271,11 +272,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					return null;
 				}
 				
+				// This extracts the UID and flags from the X-Evolution header.
+				// It may also contain user-defined flags and tags, but we don't
+				// support those right now.
 				int separator_idx = x_evolution.IndexOf ('-');
 
 				string uid_str = x_evolution.Substring (0, separator_idx);
 				string uid = Convert.ToUInt32 (uid_str, 16).ToString (); // ugh.
-				uint flags = Convert.ToUInt32 (x_evolution.Substring (separator_idx + 1), 16);
+				uint flags = Convert.ToUInt32 (x_evolution.Substring (separator_idx + 1, 4), 16);
 
 				Indexable indexable = this.GMimeMessageToIndexable (uid, message, flags);
 
@@ -312,37 +316,37 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
 
-			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
-			indexable.AddProperty (Property.NewKeyword ("fixme:account", "Local"));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:folder", this.folder_name));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "evolution"));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:account", "Local"));
+                        indexable.AddProperty (Property.NewUnsearched ("fixme:folder", this.folder_name));
 
 			GMime.InternetAddressList addrs;
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.To);
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (this.folder_name == "Sent")
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = message.GetRecipients (GMime.Message.RecipientType.Cc);
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (this.folder_name == "Sent")
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+				if (this.folder_name == "Sent"  && ia.AddressType != GMime.InternetAddressType.Group)
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (message.Sender));
 			foreach (GMime.InternetAddress ia in addrs) {
-				if (this.folder_name != "Sent")
-					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+				if (this.folder_name != "Sent"  && ia.AddressType != GMime.InternetAddressType.Group)
+					indexable.AddProperty (Property.NewUnsearched ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-	                Property flag_prop = Property.NewKeyword ("fixme:flags", flags);
+	                Property flag_prop = Property.NewUnsearched ("fixme:flags", flags);
 			flag_prop.IsMutable = true;
 			indexable.AddProperty (flag_prop);
 
@@ -373,8 +377,12 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			long offset = -1;
 
-			if (this.mbox_parser != null)
-				this.MboxLastOffset = offset = this.mbox_parser.FromOffset;
+			if (this.mbox_parser != null) {
+				offset = this.mbox_parser.FromOffset;
+
+				if (offset >= 0)
+					this.MboxLastOffset = offset;
+			}
 
 			string progress = "";
 			if (this.file_size > 0 && offset > 0) {
@@ -721,7 +729,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				indexable = new Indexable (uri);
 				indexable.Type = IndexableType.PropertyChange;
 
-				Property flag_prop = Property.NewKeyword ("fixme:flags", mi.flags);
+				Property flag_prop = Property.NewUnsearched ("fixme:flags", mi.flags);
 				flag_prop.IsMutable = true;
 				indexable.AddProperty (flag_prop);
 
@@ -759,65 +767,71 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Uri uri = CamelMessageUri (messageInfo);
 			Indexable indexable = new Indexable (uri);
 
-			indexable.Timestamp = messageInfo.Date;
+			indexable.Timestamp = messageInfo.SentDate;
 			indexable.MimeType = "message/rfc822";
 			indexable.HitType = "MailMessage";
 
-			indexable.AddProperty (Property.NewKeyword ("fixme:account",  this.imap_name));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:folder",   this.folder_name));
-			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:account",  this.imap_name));
+                        indexable.AddProperty (Property.NewUnsearched ("fixme:folder",   this.folder_name));
+			indexable.AddProperty (Property.NewUnsearched ("fixme:client", "evolution"));
 			
 			if (!have_content) {
 				indexable.AddProperty (Property.New ("dc:title", GMime.Utils.HeaderDecodePhrase (messageInfo.subject)));
-				indexable.AddProperty (Property.NewDate ("fixme:date", messageInfo.Date));
+				indexable.AddProperty (Property.NewDate ("fixme:date", messageInfo.SentDate));
 			}
 
 			GMime.InternetAddressList addrs;
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.to);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
-					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
-					indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
-					indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:to", ia.ToString (false)));
+					if (ia.AddressType != GMime.InternetAddressType.Group) {
+						indexable.AddProperty (Property.New ("fixme:to_address", ia.Addr));
+						indexable.AddProperty (Property.New ("fixme:to_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					}
 					indexable.AddProperty (Property.New ("fixme:to_name", ia.Name));
 				}
 
-				if (this.folder_name == "Sent")
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.cc);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
-					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
-					indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
-					indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:cc", ia.ToString (false)));
+					if (ia.AddressType != GMime.InternetAddressType.Group) {
+						indexable.AddProperty (Property.New ("fixme:cc_address", ia.Addr));
+						indexable.AddProperty (Property.New ("fixme:cc_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					}
 					indexable.AddProperty (Property.New ("fixme:cc_name", ia.Name));
 				}
 
-				if (this.folder_name == "Sent")
-					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+				if (this.folder_name == "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
+					indexable.AddProperty (Property.NewUnsearched ("fixme:sentTo", ia.Addr));
 			}
 			addrs.Dispose ();
 
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.from);
 			foreach (GMime.InternetAddress ia in addrs) {
 				if (!have_content) {
-					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
-					indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
-					indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					indexable.AddProperty (Property.NewUnsearched ("fixme:from", ia.ToString (false)));
+					if (ia.AddressType != GMime.InternetAddressType.Group) {
+						indexable.AddProperty (Property.New ("fixme:from_address", ia.Addr));
+						indexable.AddProperty (Property.New ("fixme:from_sanitized", StringFu.SanitizeEmail (ia.Addr)));
+					}
 					indexable.AddProperty (Property.New ("fixme:from_name", ia.Name));
 				}
 
-				if (this.folder_name != "Sent")
-					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+				if (this.folder_name != "Sent" && ia.AddressType != GMime.InternetAddressType.Group)
+					indexable.AddProperty (Property.NewUnsearched ("fixme:gotFrom", ia.Addr));
 			}
 			addrs.Dispose ();
 
                         indexable.AddProperty (Property.NewKeyword ("fixme:mlist", messageInfo.mlist));
 
-			Property flag_prop = Property.NewKeyword ("fixme:flags", messageInfo.flags);
+			Property flag_prop = Property.NewUnsearched ("fixme:flags", messageInfo.flags);
 			flag_prop.IsMutable = true;
 			indexable.AddProperty (flag_prop);
 
