@@ -37,6 +37,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 	class MailCrawler {
 		public delegate void ItemAddedHandler (FileInfo file);
 
+		private static bool Debug = false;
+
 		ArrayList roots = new ArrayList ();
 
 		Hashtable last_write_time_cache = new Hashtable ();
@@ -58,9 +60,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (last_write_time_cache.Contains (file.FullName))
 				cached_time = (DateTime) last_write_time_cache [file.FullName];
 			
-			last_write_time_cache [file.FullName] = file.LastWriteTime;
+			last_write_time_cache [file.FullName] = file.LastWriteTimeUtc;
 			
-			return cached_time < file.LastWriteTime;
+			return cached_time < file.LastWriteTimeUtc;
 		}
 
 		private void OnInotifyEvent (Inotify.Watch watch,
@@ -111,6 +113,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				string dir = (string) pending.Dequeue ();
 
 				foreach (string subdir in DirectoryWalker.GetDirectories (dir)) {
+					if (Shutdown.ShutdownRequested)
+						return;
+
 					if (Inotify.Enabled) {
 						Inotify.Subscribe (dir, OnInotifyEvent,
 								   Inotify.EventType.Create
@@ -121,7 +126,16 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					pending.Enqueue (subdir);
 				}
 
-				foreach (FileInfo file in DirectoryWalker.GetFileInfos (dir)) {
+				Stopwatch watch = new Stopwatch ();
+				if (Debug)
+					Logger.Log.Debug ("Starting watch on {0}", dir);
+				watch.Start ();
+				foreach (string path in DirectoryWalker.GetItems (dir, new DirectoryWalker.FileFilter (IsSummary))) {
+					if (Shutdown.ShutdownRequested)
+						return;
+
+					FileInfo file = new FileInfo (path);
+
 					if (file.Name == "summary") {
 						if (SummaryAddedEvent != null && FileIsInteresting (file))
 							SummaryAddedEvent (file);
@@ -133,13 +147,31 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 							MboxAddedEvent (mbox_file);
 					}
 				}
+				watch.Stop ();
+				if (Debug)
+					Logger.Log.Debug ("Crawled {0} in {1}", dir, watch);
 			}
 		}
 
+		private bool IsSummary (string path, string name)
+		{
+			if (name == "summary")
+				return true;
+
+			if (Path.GetExtension (name) == ".ev-summary")
+				return true;
+
+			return false;
+		}
+
 		public void Crawl ()
 		{
-			foreach (string root in roots)
+			foreach (string root in roots) {
+				if (Shutdown.ShutdownRequested)
+					return;
+
 				Watch (root);
+			}
 		}
 	}
 }
