@@ -80,7 +80,7 @@ namespace Beagle.Daemon {
 		public bool Write (FileAttributes attr)
 		{
 			lock (ifas) {
-				attr.LastAttrTime = DateTime.Now;
+				attr.LastAttrTime = DateTime.UtcNow;
 				return ifas.Write (attr);
 			}
 		}
@@ -137,7 +137,7 @@ namespace Beagle.Daemon {
 					return false;
 			} 
 
-			if (FileSystem.GetLastWriteTime (path) > attr.LastWriteTime)
+			if (FileSystem.GetLastWriteTimeUtc (path) > attr.LastWriteTime)
 				return false;
 
 			return true;
