@@ -222,9 +222,12 @@ namespace Beagle.Daemon {
 				if (Directory.Exists (path)) {
 					indexable.MimeType = "inode/directory";
 					indexable.NoContent = true;
-					indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
+					if (! indexable.ValidTimestamp)
+						indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
 				} else if (File.Exists (path)) {
-					indexable.Timestamp = File.GetLastWriteTimeUtc (path);
+					// Set the timestamp to the best possible estimate (if no timestamp was set by the backend)
+					if (! indexable.ValidTimestamp)
+						indexable.Timestamp = File.GetLastWriteTimeUtc (path);
 				} else {
 					Logger.Log.Warn ("No such file: {0}", path);
 					return false;
