@@ -21,6 +21,6 @@ namespace Lucene.Net.Store
 	{
 		internal System.Collections.ArrayList buffers = System.Collections.ArrayList.Synchronized(new System.Collections.ArrayList(10));
 		internal long length;
-		internal long lastModified = (System.DateTime.Now.Ticks - 621355968000000000) / 10000;
+		internal long lastModified = System.DateTime.UtcNow.Ticks;
 	}
-}
\ No newline at end of file
+}
