@@ -73,7 +73,8 @@ namespace Beagle.Daemon {
 		// 12: added year-month and year-month-day resolutions for all
 		//     date properties
 		// 13: moved source into a property
-		private const int MAJOR_VERSION = 13;
+		// 14: allow wildcard queries to also match keywords
+		private const int MAJOR_VERSION = 14;
 		private int minor_version = 0;
 
 		private string index_name;
@@ -224,6 +225,7 @@ namespace Beagle.Daemon {
 			       && Directory.Exists (LockDirectory)))
 				return false;
 
+			return true;
 			// Check the index's version number.  If it is wrong,
 			// declare the index non-existent.
 
@@ -258,8 +260,10 @@ namespace Beagle.Daemon {
 			DirectoryInfo lock_dir_info;
 			lock_dir_info = new DirectoryInfo (LockDirectory);
 			foreach (FileInfo info in lock_dir_info.GetFiles ()) {
-				if (IsDanglingLock (info))
+				if (IsDanglingLock (info)) {
+					Logger.Log.Warn ("Found a dangling index lock on {0}", info.FullName);
 					return false;
+				}
 			}
 
 			return true;
@@ -452,7 +456,7 @@ namespace Beagle.Daemon {
 		{
 			switch (type) {
 			case PropertyType.Text:    return "PropertyText";
-			case PropertyType.Keyword: return null; // wildcard keyword lookups are crack
+			case PropertyType.Keyword: return "PropertyKeyword";
 			case PropertyType.Date:    return "PropertyDate";
 			}
 
@@ -644,19 +648,19 @@ namespace Beagle.Daemon {
 
 			if (indexable.HitType != null) {
 				Property prop;
-				prop = Property.NewKeyword ("beagle:HitType", indexable.HitType);
+				prop = Property.NewUnsearched ("beagle:HitType", indexable.HitType);
 				AddPropertyToDocument (prop, primary_doc);
 			}
 
 			if (indexable.MimeType != null) {
 				Property prop;
-				prop = Property.NewKeyword ("beagle:MimeType", indexable.MimeType);
+				prop = Property.NewUnsearched ("beagle:MimeType", indexable.MimeType);
 				AddPropertyToDocument (prop, primary_doc);
 			}
 
 			if (indexable.Source != null) {
 				Property prop;
-				prop = Property.NewKeyword ("beagle:Source", indexable.Source);
+				prop = Property.NewUnsearched ("beagle:Source", indexable.Source);
 				AddPropertyToDocument (prop, primary_doc);
 			}
 
@@ -776,6 +780,7 @@ namespace Beagle.Daemon {
 				IndexReader reader;
 				reader = GetReader (PrimaryStore);
 				last_item_count = reader.NumDocs ();
+				ReleaseReader (reader);
 			}
 			return last_item_count;
 		}
@@ -1177,16 +1182,13 @@ namespace Beagle.Daemon {
 					return;
 
 				LNS.BooleanQuery p_query = new LNS.BooleanQuery ();
+				LNS.BooleanQuery s_query = new LNS.BooleanQuery ();
 
 				if (part.SearchFullText) {
 					LNS.Query subquery;
 					subquery = StringToQuery ("Text", part.Text, term_list);
-					if (subquery != null) {
-						if (primary_query == null)
-							primary_query = p_query;
-
+					if (subquery != null)
 						p_query.Add (subquery, false, false);
-					}
 
 					// FIXME: HotText is ignored for now!
 					// subquery = StringToQuery ("HotText", part.Text);
@@ -1198,17 +1200,30 @@ namespace Beagle.Daemon {
 					LNS.Query subquery;
 					subquery = StringToQuery ("PropertyText", part.Text, term_list);
 					if (subquery != null) {
-						if (primary_query == null)
-							primary_query = p_query;
-
 						p_query.Add (subquery, false, false);
-						
 						// Properties can live in either index
 						if (! only_build_primary_query)
-							secondary_query = subquery.Clone () as LNS.Query;
+							s_query.Add (subquery.Clone () as LNS.Query, false, false);
 					}
+
+					Term term;
+					term = new Term ("PropertyKeyword", part.Text);
+					// FIXME: terms are already added in term_list. But they may have been tokenized
+					// The term here is non-tokenized version. Should this be added to term_list ?
+					// term_list is used to calculate scores
+					if (term_list != null)
+						term_list.Add (term);
+					subquery = new LNS.TermQuery (term);
+					p_query.Add (subquery, false, false);
+					// Properties can live in either index
+					if (! only_build_primary_query)
+						s_query.Add (subquery.Clone () as LNS.Query, false, false);
 				}
 
+				primary_query = p_query;
+				if (! only_build_primary_query)
+					secondary_query = s_query;
+
 				return;
 			}
 
@@ -1402,15 +1417,18 @@ namespace Beagle.Daemon {
 
 			public IndexReader Reader;
 			public long Version;
+			public int Refcount;
 
 			public ReaderAndVersion (IndexReader reader, long version)
 			{
 				this.Reader = reader;
 				this.Version = version;
+				this.Refcount = 1;
 			}
 		}
 
 		static private Hashtable directory_rav_map = new Hashtable ();
+		static private Hashtable reader_rav_map = new Hashtable ();
 
 		static public LNS.IndexSearcher GetSearcher (Lucene.Net.Store.Directory directory)
 		{
@@ -1424,7 +1442,7 @@ namespace Beagle.Daemon {
 			IndexReader reader;
 			long version;
 
-			lock (directory_rav_map) {
+			lock (reader_rav_map) {
 				ReaderAndVersion rav = (ReaderAndVersion) directory_rav_map [directory];
 
 				if (rav == null) {
@@ -1432,8 +1450,10 @@ namespace Beagle.Daemon {
 					reader = IndexReader.Open (directory);
 
 					rav = new ReaderAndVersion (reader, version);
+					rav.Refcount++;
 
 					directory_rav_map [directory] = rav;
+					reader_rav_map [reader] = rav;
 
 					return reader;
 				}
@@ -1441,15 +1461,49 @@ namespace Beagle.Daemon {
 				version = IndexReader.GetCurrentVersion (directory);
 				
 				if (version != rav.Version) {
-					rav.Version = version;
-					rav.Reader.Close ();
-					rav.Reader = IndexReader.Open (directory);
-				}
-				
+					UnrefReaderAndVersion_Unlocked (rav);
+
+					reader = IndexReader.Open (directory);
+
+					rav = new ReaderAndVersion (reader, version);
+					rav.Refcount++;
+
+					directory_rav_map [directory] = rav;
+					reader_rav_map [reader] = rav;
+				} else
+					rav.Refcount++;
+
 				return rav.Reader;
 			}
 		}
 
+		static private void UnrefReaderAndVersion_Unlocked (ReaderAndVersion rav)
+		{
+			rav.Refcount--;
+
+			if (rav.Refcount == 0) {
+				rav.Reader.Close ();
+				reader_rav_map.Remove (rav.Reader);
+			}
+		}
+
+		static public void ReleaseReader (IndexReader reader)
+		{
+			lock (reader_rav_map) {
+				ReaderAndVersion rav = (ReaderAndVersion) reader_rav_map [reader];
+
+				UnrefReaderAndVersion_Unlocked (rav);
+			}
+		}
+
+		static public void ReleaseSearcher (LNS.IndexSearcher searcher)
+		{
+			IndexReader reader = searcher.GetIndexReader ();
+
+			searcher.Close ();
+			ReleaseReader (reader);
+		}
+
 		///////////////////////////////////////////////////////////////////////////////////
 
 		//
@@ -1525,6 +1579,9 @@ namespace Beagle.Daemon {
 				searcher.Close ();
 			}
 
+			ReleaseReader (primary_reader);
+			ReleaseReader (secondary_reader);
+
 			// Now assemble the hits
 			int j = 0;
 			foreach (Uri uri in primary_docs.Keys) {
@@ -1600,6 +1657,9 @@ namespace Beagle.Daemon {
 				}
 			}
 
+			ReleaseReader (primary_reader);
+			ReleaseReader (secondary_reader);
+
 			return all_hits;
 		}
 	}
