@@ -116,12 +116,9 @@ namespace Beagle.Daemon {
 
 				try {
 					reader = ExecuteReaderOrWait (command);
-				} catch (SqliteException ex) {
-					if (ex.SqliteError == SqliteError.NOTADB) {
-						Logger.Log.Warn ("Likely sqlite database version mismatch trying to read from {0}.  Purging.", db_filename);
-						create_new_db = true;
-					} else
-						throw;
+				} catch (ApplicationException ex) {
+					Logger.Log.Warn ("Likely sqlite database version mismatch trying to read from {0}.  Purging.", db_filename);
+					create_new_db = true;
 				}
 
 				if (reader != null)
@@ -150,7 +147,7 @@ namespace Beagle.Daemon {
 		{
 			SqliteConnection connection = new SqliteConnection ();
 			connection.ConnectionString = "version=" + ExternalStringsHack.SqliteVersion
-				+ ",URI=file:" + db_filename;
+				+ ",encoding=UTF-8,URI=file:" + db_filename;
 			connection.Open ();
 			return connection;
 		}
@@ -178,12 +175,9 @@ namespace Beagle.Daemon {
 				try {
 					command.ExecuteNonQuery ();
 					break;
-				} catch (SqliteException ex) {
+				} catch (SqliteBusyException ex) {
 					// FIXME: should we eventually time out?
-					if (ex.SqliteError == SqliteError.BUSY)
-						Thread.Sleep (50);
-					else
-						throw ex;
+					Thread.Sleep (50);
 				}
 			}
 			command.Dispose ();
@@ -195,11 +189,8 @@ namespace Beagle.Daemon {
 			while (reader == null) {
 				try {
 					reader = command.ExecuteReader ();
-				} catch (SqliteException ex) {
-					if (ex.SqliteError == SqliteError.BUSY)
-						Thread.Sleep (50);
-					else
-						throw ex;
+				} catch (SqliteBusyException ex) {
+					Thread.Sleep (50);
 				}
 			}
 			return reader;
@@ -210,11 +201,8 @@ namespace Beagle.Daemon {
 			while (true) {
 				try {
 					return reader.Read ();
-				} catch (SqliteException ex) {
-					if (ex.SqliteError == SqliteError.BUSY)
-						Thread.Sleep (50);
-					else
-						throw ex;
+				} catch (SqliteBusyException ex) {
+					Thread.Sleep (50);
 				}
 			}
 		}
@@ -291,6 +279,12 @@ namespace Beagle.Daemon {
 				Insert (uri, SELF_CACHE_TAG);
 		}
 
+		private bool world_readable = false;
+		public bool WorldReadable {
+			get { return this.world_readable; }
+			set { this.world_readable = value; }
+		}
+
 		public TextWriter GetWriter (Uri uri)
 		{
 			// FIXME: Uri remapping?
@@ -302,8 +296,10 @@ namespace Beagle.Daemon {
 			// We don't expect to need this again in the near future.
 			FileAdvise.FlushCache (stream);
 			
-			// Make files only readable by the owner.
-			Mono.Unix.Native.Syscall.chmod (path, (Mono.Unix.Native.FilePermissions) 384);
+			if (! world_readable) {
+				// Make files only readable by the owner.
+				Mono.Unix.Native.Syscall.chmod (path, (Mono.Unix.Native.FilePermissions) 384);
+			}
 
 			StreamWriter writer;
 			writer = new StreamWriter (stream);
