@@ -95,10 +95,11 @@ namespace Beagle.Daemon
 			
 			//Cache the query request, get a unique searchId and include in network searchRequest:
 			sreq.searchId = NetworkedBeagle.AddRequest(query);
+
 			
 			int hc = NetworkedBeagle.HopCount(query);
 			sreq.hopCount = (hc > 0) ? hc:1;
-			
+
 			log.Info("NetBeagleHandler: Starting WebService Query for " + Hostname + ":" + Port);
 			
 			ReqContext rc = new ReqContext(wsp, result, netBeagleQueryable);
@@ -120,7 +121,7 @@ namespace Beagle.Daemon
     		IQueryResult 	 result = rc.GetResult;
 			
 			int count = 0;
-			bool hitRejectsLogged = false;
+			//bool hitRejectsLogged = false;
 			
     		try
       		{	    		
@@ -133,6 +134,7 @@ namespace Beagle.Daemon
 						
 					//NetContext nc = new NetContext(wsp, resp.searchToken);						
 			   		HitResult[] hres = resp.hitResults;
+			   		ArrayList  nwhits = new ArrayList();
 		
   					for (int i = 0; i < hres.Length; i++) {
 			
@@ -140,29 +142,25 @@ namespace Beagle.Daemon
 							HitResult hr = hres[i];
 							Hit hit = new NetworkHit();
 			
-							//FIXME: Generate a random no. b/w 1 .. 99 and multiply by 1000000, and add to hr.id ?
-							hit.Id = hr.id; 
-					 
 							//[Uri Format] netbeagle://164.99.153.134:8888/searchToken?http:///....	
 							if (hr.uri.StartsWith(NetworkedBeagle.BeagleNetPrefix))
-								hit.Uri = new Uri(hr.uri);
+								hit.UriAsString = hr.uri;
 							else {							
 								string[] fragments = hr.uri.Split ('/');
 								string hostNamePort = fragments[2];										
-								hit.Uri = new Uri(NetworkedBeagle.BeagleNetPrefix + hostNamePort + "/" + resp.searchToken + "?" + hr.uri);		
+								hit.UriAsString = NetworkedBeagle.BeagleNetPrefix + hostNamePort + "/" + resp.searchToken + "?" + hr.uri;		
 							}
 														
 							hit.Type = hr.resourceType;
 							hit.MimeType = hr.mimeType;
 							hit.Source = "Network";			//hit.Source = hr.source;
-							hit.ScoreRaw = hr.scoreRaw + 0.01;
-							hit.ScoreMultiplier = hr.scoreMultiplier;
-	
+							hit.Score = hr.score;
+							
 							if (hr.properties.Length  > 0)
 							foreach (HitProperty hp in hr.properties) {
 							
 								Property p 		= Property.New(hp.PKey, hp.PVal);
-								p.IsKeyword 	= hp.IsKeyword;
+								p.IsMutable 	= hp.IsMutable;
 								p.IsSearched 	= hp.IsSearched;
 	
 								hit.AddProperty(p);
@@ -174,15 +172,14 @@ namespace Beagle.Daemon
 							//if (hr.snippet != null) 
 								//log.Debug("\nNBH: URI" + i + "=" + hr.uri + "\n     Snippet=" + hr.snippet);
 										
-							((NetworkHit)hit).context = null;
+							((NetworkHit)hit).context = new NetContext(hr.hashCode);
 							  						
 							//Add NetBeagleQueryable instance
 							hit.SourceObject = iq;
+							hit.SourceObjectName = ((NetworkedBeagle)iq).Name; 
 						
-							if ((! result.Add (hit)) && (! hitRejectsLogged)) {
-								hitRejectsLogged = true;
-								log.Info("NetBeagleHandler: Network Hits rejected by HitRegulator. Too many Hits!");
-							}
+							nwhits.Add(hit); 
+
 							count++;							
 						}
 						catch (Exception ex2) {
@@ -191,16 +188,31 @@ namespace Beagle.Daemon
 							//log.Error ("Exception StackTrace: " + ex.StackTrace);
 		 				}				
 					}  //end for 
-					
+
+					if (nwhits.Count > 0) 
+						result.Add (nwhits);
+/*				
+					if ((! result.Add (nwhits)) && (! hitRejectsLogged)) 
+					{
+						hitRejectsLogged = true;
+						log.Info("NetBeagleHandler: Network Hits rejected by HitRegulator. Too many Hits!");
+					}
+*/					
 					log.Info("NetBeagleHandler: DoQueryResponseHandler() Got {0} result(s) from Index {1} from Networked Beagle at {2}", count, resp.firstResultIndex, wsp.Hostname + ":" + wsp.Port); 		   		
 			
 					int index = resp.firstResultIndex + resp.numResults;			
 					if (index  < resp.totalResults) {
 					
-						log.Debug("NetBeagleHandler: DoQueryResponseHandler() invoking GetMoreResults with index: " + index);
+						log.Debug("NetBeagleHandler: DoQueryResponseHandler() invoking GetResults with index: " + index);
 						
 						string searchToken = resp.searchToken;									
-						IAsyncResult ar2 = wsp.BeginGetMoreResults(searchToken, index, NetBeagleHandler.DoQueryResponseHandler, rc);
+						
+						GetResultsRequest req = new GetResultsRequest();
+						req.startIndex = index;
+						req.searchToken = searchToken;						
+											
+						IAsyncResult ar2;					
+						ar2 = wsp.BeginGetResults(req, NetBeagleHandler.DoQueryResponseHandler, rc);						
 						
 						return;						
 					}										
@@ -260,25 +272,19 @@ namespace Beagle.Daemon
 		}		
 	}
 
-/*
+
 	public class NetContext {
 
-		private string token;		
-		private BeagleWebService wsp;
+		private int _hashCode;
 					
-		public NetContext(BeagleWebService wsp, string token)
+		public NetContext(int hashCode)
 		{
-			this.wsp = wsp;
-			this.token = token;
+			this._hashCode = hashCode;
 		}		
 		
-		public BeagleWebService  proxy {
-			get { return wsp; }
-		}		
-		
-		public string searchToken {
-			get {return token; }
-		} 		
+		public int hashCode {
+			get { return _hashCode; }
+		}			
 	}
-*/		
+	
 }
