@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2003 Marco Pesenti Gritti
- *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2003, 2004, 2005 Christian Persch
  *  Copyright (C) 2003, 2004 Lee Willis
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -17,14 +17,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-beagle-extension.c,v 1.3 2004/07/27 09:20:24 orph Exp $
+ *  $Id: ephy-beagle-extension.c,v 1.5 2005/12/02 17:01:36 joeshaw Exp $
  */
 
 /*
   This is all copied from Dashboard's Epiphany Extension.
 */
 
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -32,10 +31,14 @@
 #include "ephy-beagle-extension.h"
 
 #include <epiphany/ephy-extension.h>
+#include <epiphany/ephy-embed-factory.h>
 #include <epiphany/ephy-embed-persist.h>
 #include <epiphany/ephy-shell.h>
 #include <epiphany/ephy-node.h>
 #include <epiphany/ephy-bookmarks.h>
+#include <epiphany/ephy-window.h>
+#include <epiphany/ephy-embed.h>
+#include <epiphany/ephy-tab.h>
 
 #include <gmodule.h>
 
@@ -43,60 +46,12 @@
 
 struct EphyBeagleExtensionPrivate
 {
+	gpointer dummy;
 };
 
-#define EPIPHANY_FRONTEND_IDENTIFIER	"Web Browser"
-
-static void ephy_beagle_extension_class_init	(EphyBeagleExtensionClass *klass);
-static void ephy_beagle_extension_iface_init	(EphyExtensionIface *iface);
-static void ephy_beagle_extension_init	(EphyBeagleExtension *extension);
-
 static GObjectClass *parent_class = NULL;
-
 static GType type = 0;
 
-GType
-ephy_beagle_extension_get_type (void)
-{
-	return type;
-}
-
-GType
-ephy_beagle_extension_register_type (GTypeModule *module)
-{
-	static const GTypeInfo our_info =
-	{
-		sizeof (EphyBeagleExtensionClass),
-		NULL, /* base_init */
-		NULL, /* base_finalize */
-		(GClassInitFunc) ephy_beagle_extension_class_init,
-		NULL,
-		NULL, /* class_data */
-		sizeof (EphyBeagleExtension),
-		0, /* n_preallocs */
-		(GInstanceInitFunc) ephy_beagle_extension_init
-	};
-
-	static const GInterfaceInfo extension_info =
-	{
-		(GInterfaceInitFunc) ephy_beagle_extension_iface_init,
-		NULL,
-		NULL
-	};
-
-	type = g_type_module_register_type (module,
-					    G_TYPE_OBJECT,
-					    "EphyBeagleExtension",
-					    &our_info, 0);
-
-	g_type_module_add_interface (module,
-				     type,
-				     EPHY_TYPE_EXTENSION,
-				     &extension_info);
-
-	return type;
-}
-
 static void
 load_status_cb (EphyTab *tab,
 		GParamSpec *pspec,
@@ -131,9 +86,9 @@ load_status_cb (EphyTab *tab,
 		page_title = ephy_tab_get_title(tab);
 
 		/* Get the page content. */
-		persist = EPHY_EMBED_PERSIST (ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		persist = EPHY_EMBED_PERSIST (ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 		ephy_embed_persist_set_embed (persist, embed);
-		ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW);
+		ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW);
 		content = ephy_embed_persist_to_string (persist);
 		g_object_unref (persist);
 
@@ -167,23 +122,19 @@ load_status_cb (EphyTab *tab,
 }
 
 static void
-tab_added_cb (GtkWidget *notebook,
-	      EphyTab *tab,
-	      EphyBeagleExtension *extension)
+impl_attach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyTab *tab)
 {
-	g_return_if_fail (EPHY_IS_TAB (tab));
-
 	g_signal_connect_after (tab, "notify::load-status",
 				G_CALLBACK (load_status_cb), extension);
 }
 
 static void
-tab_removed_cb (GtkWidget *notebook,
-		EphyTab *tab,
-		EphyBeagleExtension *extension)
+impl_detach_tab (EphyExtension *extension,
+		 EphyWindow *window,
+		 EphyTab *tab)
 {
-	g_return_if_fail (EPHY_IS_TAB (tab));
-
 	g_signal_handlers_disconnect_by_func
 		(tab, G_CALLBACK (load_status_cb), extension);
 }
@@ -192,28 +143,12 @@ static void
 impl_attach_window (EphyExtension *ext,
 		    EphyWindow *window)
 {
-	GtkWidget *notebook;
-
-	notebook = ephy_window_get_notebook (window);
-
-	g_signal_connect_after (notebook, "tab_added",
-				G_CALLBACK (tab_added_cb), ext);
-	g_signal_connect_after (notebook, "tab_removed",
-				G_CALLBACK (tab_removed_cb), ext);
 }
 
 static void
 impl_detach_window (EphyExtension *ext,
 		    EphyWindow *window)
 {
-	GtkWidget *notebook;
-
-	notebook = ephy_window_get_notebook (window);
-
-	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_added_cb), ext);
-	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_removed_cb), ext);
 }
 
 static void
@@ -227,6 +162,8 @@ ephy_beagle_extension_iface_init (EphyExtensionIface *iface)
 {
 	iface->attach_window = impl_attach_window;
 	iface->detach_window = impl_detach_window;
+	iface->attach_tab = impl_attach_tab;
+	iface->detach_tab = impl_detach_tab;
 }
 
 static void
@@ -234,7 +171,49 @@ ephy_beagle_extension_class_init (EphyBeagleExtensionClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
+	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
 
 	g_type_class_add_private (object_class, sizeof (EphyBeagleExtensionPrivate));
 }
+
+GType
+ephy_beagle_extension_get_type (void)
+{
+	return type;
+}
+
+GType
+ephy_beagle_extension_register_type (GTypeModule *module)
+{
+	static const GTypeInfo our_info =
+	{
+		sizeof (EphyBeagleExtensionClass),
+		NULL, /* base_init */
+		NULL, /* base_finalize */
+		(GClassInitFunc) ephy_beagle_extension_class_init,
+		NULL,
+		NULL, /* class_data */
+		sizeof (EphyBeagleExtension),
+		0, /* n_preallocs */
+		(GInstanceInitFunc) ephy_beagle_extension_init
+	};
+
+	static const GInterfaceInfo extension_info =
+	{
+		(GInterfaceInitFunc) ephy_beagle_extension_iface_init,
+		NULL,
+		NULL
+	};
+
+	type = g_type_module_register_type (module,
+					    G_TYPE_OBJECT,
+					    "BeagleExtension",
+					    &our_info, 0);
+
+	g_type_module_add_interface (module,
+				     type,
+				     EPHY_TYPE_EXTENSION,
+				     &extension_info);
+
+	return type;
+}
