@@ -1,5 +1,5 @@
 using System;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Search.Tiles {
 
@@ -24,7 +24,6 @@ namespace Search.Tiles {
 			Group = TileGroup.Audio;
 
 			// FIXME: Show album art if any. Needs implementation in Beagle.Util
-			Icon = WidgetFu.LoadMimeIcon (Hit ["beagle:MimeType"], 32);
 
 			string title = Hit.GetFirstProperty ("fixme:title");
 			if (title != null && title != "")
@@ -34,7 +33,7 @@ namespace Search.Tiles {
 			if (artist != null && artist != "")
 				Description = artist;
 
-			AddAction (new TileAction (Catalog.GetString ("Add to Library"), AddToLibrary));
+//			AddAction (new TileAction (Catalog.GetString ("Add to Library"), AddToLibrary));
 		}
 
 		// FIXME: Check if Banshee exists and supports this?
