@@ -2,6 +2,7 @@ using System;
 using System.Diagnostics;
 using System.Collections;
 using Mono.Unix;
+using Beagle.Util;
 
 namespace Search.Tiles {
 
@@ -26,12 +27,12 @@ namespace Search.Tiles {
 		{
 			Group = TileGroup.Conversations;
 
-			subject.LabelProp = Catalog.GetString ("IM Conversation");
-			from.LabelProp = "<b>" + hit.GetFirstProperty ("fixme:speakingto") + "</b>";
+			Subject.LabelProp = Catalog.GetString ("IM Conversation");
+			From.LabelProp = "<b>" + hit.GetFirstProperty ("fixme:speakingto") + "</b>";
 
 			try {
 				Timestamp = Utils.ParseTimestamp (hit.GetFirstProperty ("fixme:starttime"));
-				date.LabelProp = Utils.NiceShortDate (Timestamp);
+				Date.LabelProp = Utils.NiceShortDate (Timestamp);
 			} catch {}
 		}
 
@@ -51,16 +52,6 @@ namespace Search.Tiles {
 
 		protected override void LoadIcon (Gtk.Image image, int size)
 		{
-			if (size > 32) {
-				// FIXME: We do not respect the icon size request
-				Gdk.Pixbuf icon = LoadBuddyIcon ();
-				
-				if (icon != null) {
-					image.Pixbuf = icon;
-					return;
-				}
-			}
-
 			Hashtable icons = (Hashtable)all_icons[size];
 			if (icons == null)
 				all_icons[size] = icons = IconsForSize (size);
@@ -86,8 +77,9 @@ namespace Search.Tiles {
 		{
 			DetailsPane details = new DetailsPane ();
 
+			details.Icon.Pixbuf = LoadBuddyIcon ();			
 			details.AddLabelPair (Catalog.GetString ("Name:"), FromLabel.Text, 0, 1);
-			details.AddLabelPair (Catalog.GetString ("Date Received:"), DateLabel.Text, 1, 1);
+			details.AddLabelPair (Catalog.GetString ("Date Received:"), Utils.NiceLongDate (Timestamp), 1, 1);
 
 			GotSnippet += SetSubject;
 			details.AddSnippet (2, 1);
@@ -97,21 +89,21 @@ namespace Search.Tiles {
 
 		private void SetSubject (string snippet)
 		{
-			subject.Markup = snippet;
+			Subject.Markup = snippet;
 		}
 
 		public override void Open ()
 		{
-			Process p = new Process ();
-			p.StartInfo.UseShellExecute = true;
-			p.StartInfo.FileName = "beagle-imlogviewer";
-			p.StartInfo.Arguments = String.Format ("--client \"{0}\" --highlight-search \"{1}\" {2}",
-							       Hit ["fixme:client"], Query.QuotedText, Hit.Uri.LocalPath);
+			SafeProcess p = new SafeProcess ();
+			p.Arguments = new string [] { "beagle-imlogviewer",
+						      "--client", Hit ["fixme:client"],
+						      "--highlight-search", Query.QuotedText,
+						      Hit.Uri.LocalPath };
 
 			try {
 				p.Start ();
 			} catch (Exception e) {
-				Console.WriteLine ("Unable to run {0}: {1}", p.StartInfo.FileName, e.Message);
+				Console.WriteLine ("Unable to run {0}: {1}", p.Arguments [0], e.Message);
 			}
 		}
 	}
