@@ -3,7 +3,7 @@ using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text.RegularExpressions;
 using System.Diagnostics;
-using Mono.Posix;
+using Mono.Unix;
 
 using Gtk;
 using Beagle;
@@ -15,11 +15,13 @@ namespace Search.Tiles {
 
 		public Tile (Hit hit, Query query) : base ()
 		{
-			ModifyBg (Gtk.StateType.Normal, Style.Base (Gtk.StateType.Normal));
 			AboveChild = true;
+			AppPaintable = true;
 			CanFocus = true;
 
 			this.hit = hit;
+			this.timestamp = hit.Timestamp;
+			this.score = hit.Score;
 			this.query = query;
 			this.group = TileGroup.Documents;
 
@@ -30,9 +32,9 @@ namespace Search.Tiles {
 			hbox.BorderWidth = 2;
 			hbox.Show ();
 
-			image = new Gtk.Image ();
-			image.NoShowAll = true;
-			hbox.PackStart (image, false, false, 0);
+			icon = new Gtk.Image ();
+			icon.Show ();
+			HBox.PackStart (icon, false, false, 0);
 
 			Add (hbox);
 		}
@@ -47,7 +49,7 @@ namespace Search.Tiles {
 			get { return query; }
 		}
 
-		protected TileGroup group;
+		private TileGroup group;
 		public TileGroup Group {
 			get { return group; }
 			set { group = value; }
@@ -58,36 +60,44 @@ namespace Search.Tiles {
 			get { return hbox; }
 		}
 
-		private Gtk.Image image;
-		public Gdk.Pixbuf Icon {
-			get { return image.Pixbuf; }
-			set {
-				image.Pixbuf = value;
-				UpdateIcon ();
-			}
+		private Gtk.Image icon;
+		public Gtk.Image Icon {
+			get { return icon; }
+			set { icon = value; }
 		}
 
+		private string title;
+		public virtual string Title {
+			get { return title; }
+			set { title = value; }
+		}
+
+		private DateTime timestamp;
+		public virtual DateTime Timestamp {
+			get { return timestamp; }
+			set { timestamp = value; }
+		}
+
+		private double score;
+		public virtual double Score {
+			get { return score; }
+			set { score = value; }
+		}
+
+		protected bool EnableOpenWith = false;
+
 		static Gtk.TargetEntry[] targets = new Gtk.TargetEntry[] {
 			new Gtk.TargetEntry ("text/uri-list", 0, 0)
 		};
 
-		public event EventHandler Selected, Deselected;
-
-		private void UpdateIcon ()
-		{
-			if (image.Pixbuf != null) {
-				image.Show ();
-			} else {
-				image.Hide ();
-			}
-		}
+		public event EventHandler Selected;
 
 		protected override void OnDragBegin (Gdk.DragContext context)
 		{
-			if (!image.Visible)
+			if (!icon.Visible)
 				return;
 
-			WidgetFu.SetDragImage (context, image);
+			WidgetFu.SetDragImage (context, icon);
 		}
 
 		protected override void OnDragDataGet (Gdk.DragContext dragContext,
@@ -125,10 +135,17 @@ namespace Search.Tiles {
 
 		protected override bool OnExposeEvent (Gdk.EventExpose evt)
 		{
+			if (!IsDrawable)
+				return false;
+
+			GdkWindow.DrawRectangle (Style.BaseGC (State), true,
+						 evt.Area.X, evt.Area.Y,
+						 evt.Area.Width, evt.Area.Height);
+
 			if (base.OnExposeEvent (evt))
 				return true;
 
-			if (IsDrawable && HasFocus) {
+			if (HasFocus) {
 				Gdk.Rectangle alloc = Allocation;
 				int focusPad = (int)StyleGetProperty ("focus-padding");
 
@@ -137,12 +154,24 @@ namespace Search.Tiles {
 				int width = alloc.Width - 2 * (focusPad + Style.Xthickness);
 				int height = alloc.Height - 2 * (focusPad + Style.Ythickness);
 				Style.PaintFocus (Style, GdkWindow, State, evt.Area, this,
-						  "button", x, y, width, height);
+						  null, x, y, width, height);
 			}
 
 			return false;
 		}
 
+		///////////////////////////////////////////////////
+
+		public ArrayList actions = new ArrayList ();
+		public ICollection Actions {
+			get { return actions; }
+		}
+
+		protected void AddAction (TileAction action)
+		{
+			actions.Add (action);
+		}
+
 		private void ShowPopupMenu ()
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
@@ -150,6 +179,16 @@ namespace Search.Tiles {
 			ActionMenuItem mi = new ActionMenuItem (new TileAction (Catalog.GetString ("Open"), Stock.Open, Open));
 			menu.Append (mi);
 
+#if NOT_YET
+			// FIXME: Disabled until we have a reasonable workaround
+			// for lower gtk# versions.
+			if (EnableOpenWith) {
+				OpenWithMenu owm = new OpenWithMenu (Hit ["beagle:MimeType"]);
+				owm.ApplicationActivated += OpenWith;
+				owm.AppendToMenu (menu);
+			}
+#endif
+
 			if (Actions.Count > 0) {
 				SeparatorMenuItem si = new SeparatorMenuItem ();
 				menu.Append (si);
@@ -164,6 +203,8 @@ namespace Search.Tiles {
 			menu.Popup ();
 		}
 
+		///////////////////////////////////////////////////
+
 		protected override bool OnButtonPressEvent (Gdk.EventButton b)
 		{
 			GrabFocus ();
@@ -188,13 +229,6 @@ namespace Search.Tiles {
 			return base.OnFocusInEvent (f);
 		}
 
-		protected override bool OnFocusOutEvent (Gdk.EventFocus f)
-		{
-			if (Deselected != null && !IsFocus)
-				Deselected (this, EventArgs.Empty);
-			return base.OnFocusOutEvent (f);
-		}
-
 		protected override bool OnKeyPressEvent (Gdk.EventKey k)
 		{
 			if (k.Key == Gdk.Key.Return || k.Key == Gdk.Key.KP_Enter) {
@@ -207,11 +241,22 @@ namespace Search.Tiles {
 			return base.OnKeyPressEvent (k);
 		}
 
+		protected virtual void LoadIcon (Gtk.Image image, int size)
+		{
+			image.Pixbuf = WidgetFu.LoadMimeIcon (hit.MimeType, size);
+		}
+
+		string snippet;
+
 		protected void RequestSnippet ()
 		{
-			SnippetRequest sreq = new SnippetRequest (query, hit);
-			sreq.RegisterAsyncResponseHandler (typeof (SnippetResponse), SnippetResponseReceived);
-			sreq.SendAsync ();
+			if (snippet != null)
+				EmitGotSnippet ();
+			else {
+				SnippetRequest sreq = new SnippetRequest (query, hit);
+				sreq.RegisterAsyncResponseHandler (typeof (SnippetResponse), SnippetResponseReceived);
+				sreq.SendAsync ();
+			}
 		}
 
 		private void SnippetResponseReceived (ResponseMessage response)
@@ -224,34 +269,45 @@ namespace Search.Tiles {
 			// other tags escaped.
 
 			// FIXME: hacky, fix the snippeting in the daemon
-			string snippet = GLib.Markup.EscapeText (((SnippetResponse)response).Snippet);
+			snippet = GLib.Markup.EscapeText (((SnippetResponse)response).Snippet);
 			snippet = Regex.Replace (snippet, "&lt;font color=&quot;.*?&quot;&gt;&lt;b&gt;(.*?)&lt;/b&gt;&lt;/font&gt;", "<b>$1</b>");
 
-			if (snippet != null && snippet != "")
-				GotSnippet (snippet, true);
+			EmitGotSnippet ();
 		}
 
-		protected virtual void GotSnippet (string snippet, bool found)
+		private void EmitGotSnippet ()
 		{
+			if (snippet != null && snippet != "" && GotSnippet != null)
+				GotSnippet (snippet);
 		}
 
-		protected virtual Gtk.Widget GetDetails ()
+		public delegate void GotSnippetHandler (string snippet);
+		public event GotSnippetHandler GotSnippet;
+
+		protected virtual DetailsPane GetDetails ()
 		{
 			return null;
 		}
 
+		DetailsPane details;
 		public Gtk.Widget Details {
-			get { return GetDetails (); }
-		}
+			get {
+				if (details == null) {
+					details = GetDetails ();
+					if (details != null) {
+						if (details.Icon.Pixbuf == null)
+							LoadIcon (details.Icon, 128);
 
-		public ArrayList actions = new ArrayList ();
-		public ICollection Actions {
-			get { return actions; }
-		}
-
-		protected void AddAction (TileAction action)
-		{
-			actions.Add (action);
+						if (details.Snippet != null) {
+							GotSnippet += details.GotSnippet;
+							RequestSnippet ();
+						}
+						
+						details.Show ();
+					}
+				}
+				return details;
+			}
 		}
 
 		public virtual void Open ()
@@ -259,22 +315,25 @@ namespace Search.Tiles {
 			System.Console.WriteLine ("Warning: Open method not implemented for this tile type");
 		}
 
+#if NOT_YET
+		private void OpenWith (Gnome.Vfs.MimeApplication mime_application)
+		{
+			GLib.List uri_list = new GLib.List (typeof (string));
+			uri_list.Append (Hit.UriAsString);
+			mime_application.Launch (uri_list);
+		}
+#endif
+
 		protected void OpenFromMime (Hit hit)
 		{
-			OpenFromMime (hit, null, null, false);
-		}
-
-		protected void OpenFromMime (Hit hit, string command_fallback,
-					     string args_fallback, bool expects_uris_fallback)
-		{
-			string argument;
-			string command = command_fallback;
-			bool expects_uris = expects_uris_fallback;
+			string command = null, item;
+			bool expects_uris = false;
 
 			// FIXME: This is evil.  Nautilus should be handling
 			// inode/directory, not just x-directory/normal
 			if (hit.MimeType == "inode/directory")
 				hit.MimeType = "x-directory/normal";
+
 #if ENABLE_DESKTOP_LAUNCH
 			command = "desktop-launch";
 			expects_uris = true;
@@ -291,38 +350,47 @@ namespace Search.Tiles {
 				return;
 			}
 
-			if (args_fallback != null)
-				argument = args_fallback;
-			else 
-				argument = "";			
-
-			if (expects_uris) {
-				argument = String.Format ("{0} '{1}'", argument, hit.Uri);
-			} else {
-				argument = String.Format ("{0} {1}", argument, hit.PathQuoted);
-			}
+			if (expects_uris)
+				item = UriFu.UriToSerializableString (hit.Uri);
+			else
+				item = hit.Path;
 
 			// Sometimes the command is 'quoted'
 			if (command.IndexOf ('\'') == 0 && command.LastIndexOf ('\'') == command.Length - 1)
 				command = command.Trim ('\'');
 
 			// This won't work if a program really has a space in
-			// the filename, but I think other things would break
-			// with that too, and in practice it doesn't seem to
+			// the command filename, but I think other things would
+			// break with that too, and in practice it doesn't seem to
 			// happen.
+			//
+			// A bigger issue is that the arguments are split up by
+			// spaces, so quotation marks used to indicate a single
+			// entry in the argv won't work.  This probably should
+			// be fixed.
+			string[] arguments = null;
 			int idx = command.IndexOf (' ');
 			if (idx != -1) {
-				argument = String.Format ("{0} {1}", command.Substring (idx + 1), argument);
+				arguments = command.Substring (idx + 1).Split (' ');
 				command = command.Substring (0, idx);
 			}
 
-			Console.WriteLine ("Cmd: {0}", command);
-			Console.WriteLine ("Arg: {0}", argument);
+			string[] argv;
+			if (arguments == null)
+				argv = new string [] { command, item };
+			else {
+				argv = new string [arguments.Length + 2];
+				argv [0] = command;
+				argv [argv.Length - 1] = item;
+				Array.Copy (arguments, 0, argv, 1, arguments.Length);
+			}
 
-			Process p = new Process ();
-			p.StartInfo.UseShellExecute = false;
-			p.StartInfo.FileName = command;
-			p.StartInfo.Arguments = argument;
+			Console.WriteLine ("Cmd: {0}", command);
+			Console.WriteLine ("Arg: {0}", String.Join (" ", argv, 1, argv.Length - 2));
+			Console.WriteLine ("Itm: {0}", item);
+
+			SafeProcess p = new SafeProcess ();
+			p.Arguments = argv;
 
 			try {
 				p.Start ();
@@ -339,10 +407,8 @@ namespace Search.Tiles {
 		public void OpenFromUri (string uri)
                 {
 #if ENABLE_DESKTOP_LAUNCH
-			Process p = new Process ();
-			p.StartInfo.UseShellExecute = false;
-			p.StartInfo.FileName = "desktop-launch";
-			p.StartInfo.Arguments = uri;
+			SafeProcess p = new SafeProcess ();
+			p.Arguments = new string[] { "desktop-launch", uri };
 
 			try {
 				p.Start ();
